Package: SEMPLR
Title: SNP Effect Matrix Pipeline in R
Version: 1.0.0
Authors@R: c(
    person("Grace", "Kenney", 
    email = "kenney.grace6@gmail.com", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0009-0009-6308-3150")),
    person("Douglas", "Phanstiel", 
    role = c("aut")),
    person("NIH NIGMS", role = "fnd"),
    person("NSF GRFP", role = "fnd"))
Description: SEMPLR computes transcription factor binding affinity scores for 
    genomic positions and genetic variants. Scores are computed from SNP Effect 
    Matrices (SEMs) produced by SEMpl. 223 pre-computed SEMs are included
    with the package or custom sets can be provided. Enrichment can be tested 
    among sets of genomic positions to determine if transcription factor 
    binding events occur more often than expected. Comparing binding affinity 
    scores between alleles can reveal differences in transcription factor 
    binding with genetic variation. This package also includes several 
    visualization functions to view scores both on the motif and 
    variant/position level.
Imports: BiocGenerics, Biostrings, GenomeInfoDb, AnnotationDbi,
        ggplot2, ggrepel, VariantAnnotation, GenomicRanges,
        GenomicFeatures, data.table, methods, scales, S4Vectors, stats,
        rlang, stringi, universalmotif, Rcpp, ggtree
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, BiocStyle, devtools, testthat (>= 3.0.0),
        IRanges, BSgenome.Hsapiens.UCSC.hg38,
        BSgenome.Hsapiens.UCSC.hg19, TxDb.Hsapiens.UCSC.hg38.knownGene,
        org.Hs.eg.db
VignetteBuilder: knitr
License: MIT + file LICENSE
Config/testthat/edition: 3
URL: https://github.com/grkenney/SEMPLR,
        https://grkenney.github.io/SEMPLR
BugReports: https://www.github.com/grkenney/SEMPLR/issues
biocViews: MotifAnnotation, Transcription, SNP, GenomicVariation
LazyData: true
LinkingTo: Rcpp
git_url: https://git.bioconductor.org/packages/SEMPLR
git_branch: RELEASE_3_23
git_last_commit: 61bf45c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:39:44 UTC; biocbuild
Author: Grace Kenney [aut, cre] (ORCID:
    <https://orcid.org/0009-0009-6308-3150>),
  Douglas Phanstiel [aut],
  NIH NIGMS [fnd],
  NSF GRFP [fnd]
Maintainer: Grace Kenney <kenney.grace6@gmail.com>
Depends: R (>= 4.1.0)
