# Generated by roxygen2: do not edit by hand

export(SEMScores)
export(SNPEffectMatrix)
export(SNPEffectMatrixCollection)
export(convertSEMsToPPMs)
export(enrichSEMs)
export(enrichmentSets)
export(getBaseline)
export(getRangeSeqs)
export(getRanges)
export(getSEM)
export(getSEMId)
export(getSEMs)
export(loadSEMCollection)
export(plotEnrich)
export(plotSEM)
export(plotSEMMotifs)
export(plotSEMVariants)
export(scoreBinding)
export(scoreVariants)
export(scores)
export(semData)
exportClasses(SNPEffectMatrix)
exportClasses(SNPEffectMatrixCollection)
exportMethods(getBaseline)
exportMethods(getRanges)
exportMethods(getSEM)
exportMethods(getSEMId)
exportMethods(getSEMs)
exportMethods(scores)
exportMethods(semData)
exportMethods(show)
import(ggplot2)
import(methods)
importClassesFrom(GenomicRanges,GRanges)
importClassesFrom(VariantAnnotation,VRanges)
importFrom(GenomicRanges,GRanges)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,mcols)
importFrom(VariantAnnotation,VRanges)
importFrom(data.table,data.table)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(rlang,":=")
useDynLib(SEMPLR, .registration = TRUE)
