#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom data.table data.table
#' @importFrom Rcpp sourceCpp
#' @importFrom rlang :=
#' @useDynLib SEMPLR, .registration = TRUE
## usethis namespace: end
NULL
